Multimodal Network Diffusion
==============================

Repository for *Multimodal Network Diffusion Predicts Future Disease-Gene-Chemical Associations*
## Content
 - [Download code](#download-code)
 - [Installation](#installation)
 - [Run tutorial](#run-tutorial)
 - [Run experiments](#run-experiments)
 - [Project organization](#run-tutorial)

## Download code
```bash
git clone https://some.where.out.there
cd multimodal-network-diffusion
```

## Installation

### Requirements
- [Anaconda](https://www.anaconda.com/) or [MiniConda](https://conda.io/miniconda.html)
- For Mac: [Julia 0.5.2](https://julialang.org/downloads/oldreleases.html)
- For Linux: [Julia 0.6.3](https://julialang.org/downloads/)

- Add Julia path in system variable `PATH` 
Mac commands are below. You might need to change the Julia path if you install in a different location.
```bash
echo "export PATH=\"/Applications/Julia-0.5.app/Contents/Resources/julia/bin:\$PATH\"" >> "$HOME/.bash_profile" 
source ~/.bash_profile
```

### Install
```bash
git clone 
cd multimodal-network-diffusion
./install.sh
```
*One 
## Run tutorial


### 1. Open Jupyter notebook in `./notebooks/`
```bash
cd notebooks/
jupyter notebook
```
### 2. Click on `Prospective_example.ipynb`
### 3. Choose the kernel of the `Diffusion2018` environment
#### (1) Choose `Python [Diffusion2018]`  when being asked to choose kernel
#### or (2) (In the menu bar of the newly opened page) Kernel -> Change kernel -> Python [Diffusion2018]
*The kernel name could be `Python [Diffusion2018]` or `Python [conda env:Diffusion2018]` in our tests.*
### 4. Run each cell

### (If you don't see `Python [Diffusion2018]` in Jupyter Notebook) Enable environment for Jupyter Notebook
#### 1. Activate environment (skip it if you already activated)
```bash
source activate Diffusion2018
```
#### 2. Register the environment for Jupyter Notebook
```bash
python -m ipykernel install --user --name Diffusion2018 --display-name "Python [Diffusion2018]"
```
## Run experiments

### 1. Activate environment
```bash
source activate Diffusion2018
```

### 2. Example of running prospective experiment of disease-gene prediction
```bash
cd src
python runProspective.py --e DG
```

Project Organization
------------

    ├── LICENSE
    ├── Makefile           <- Makefile with commands like `make data` or `make train`
    ├── README.md          <- The top-level README for developers using this project.
    ├── data
    │   ├── external       <- Data from third party sources.
    │   ├── interim        <- Intermediate data that has been transformed.
    │   ├── processed      <- The final, canonical data sets for modeling.
    │   └── raw            <- The original, immutable data dump.
    │
    ├── docs               <- A default Sphinx project; see sphinx-doc.org for details
    │
    ├── models             <- Trained and serialized models, model predictions, or model summaries
    │
    ├── notebooks          <- Jupyter notebooks. Naming convention is a number (for ordering),
    │                         the creator's initials, and a short `-` delimited description, e.g.
    │                         `1.0-jqp-initial-data-exploration`.
    │
    ├── references         <- Data dictionaries, manuals, and all other explanatory materials.
    │
    ├── reports            <- Generated analysis as HTML, PDF, LaTeX, etc.
    │   └── figures        <- Generated graphics and figures to be used in reporting
    │
    ├── requirements.txt   <- The requirements file for reproducing the analysis environment, e.g.
    │                         generated with `pip freeze > requirements.txt`
    │
    ├── setup.py           <- makes project pip installable (pip install -e .) so src can be imported
    ├── src                <- Source code for use in this project.
    │   ├── __init__.py    <- Makes src a Python module
    │   │
    │   ├── data           <- Scripts to download or generate data
    │   │   └── make_dataset.py
    │   │
    │   ├── features       <- Scripts to turn raw data into features for modeling
    │   │   └── build_features.py
    │   │
    │   ├── models         <- Scripts to train models and then use trained models to make
    │   │   │                 predictions
    │   │   ├── predict_model.py
    │   │   └── train_model.py
    │   │
    │   └── visualization  <- Scripts to create exploratory and results oriented visualizations
    │       └── visualize.py
    │
    └── tox.ini            <- tox file with settings for running tox; see tox.testrun.org


--------

<p><small>Project based on the <a target="_blank" href="https://drivendata.github.io/cookiecutter-data-science/">cookiecutter data science project template</a>. #cookiecutterdatascience</small></p>

> Written with [StackEdit](https://stackedit.io/).
<!--stackedit_data:
eyJoaXN0b3J5IjpbMTU1MjkxNjA4OCwxOTQ0NzkxMjMxLC02OD
Q0MTY1NTAsLTk4MjI0MDkxNiwtNzMwMjU3MzcyLC04ODMxMDUz
NjQsLTEyMTAwNjg0MTYsLTExMTIxMDk2ODIsLTk4MzM5MzEwNy
wtNTUwNDY0MTM0XX0=
-->