'''
Created on Aug 26, 2017

@author: Chih-Hsu Lin

This script is for running prospective experiments across different networks
'''


import sys
import os
import paths
from GraphRepresentation5 import GraphRepresentation5
from AptRank import AptRank
from GraphBasedDiffusion import GraphBasedDiffusion
from RandomWalkDiffusion import RandomWalkDiffusion
from prospective import prospective
from IPython import embed
from time import time
import argparse
from utility import setNetworks, generateDegree1net, parseArguments, printPerformance, plotMultipleROC,plotMultipleAUCbootstrap_violinplot,plotMultiplePRcurve
import pandas as pd
from collections import defaultdict as ddict

if __name__ == '__main__':
    start = time()
    previous = time()
    args = parseArguments()
    if(type(args['mapping']) == str):
        mappingFile = args['mapping']
    else:
        mappingFile = args['mapping'][0]
    print mappingFile
    if(type(args['data']) == str):
        dataDir = args['data']
    else:
        dataDir = args['data'][0]
    print dataDir
    ##########################################################################
    # Experiment setting
    ##########################################################################
    if(type(args['experiment']) == str):
        experiment = args['experiment']
    else:
        experiment = args['experiment'][0]
    print experiment

    networkNames, conns, selectedQueryModes, selectedPredictionModes = setNetworks(
        2016, 'Prospective', experiment, mappingFile, dataDir)
    # Defining the algorithm
    algorithms = [GraphBasedDiffusion, RandomWalkDiffusion, AptRank]
    algorithmNames = ['GID', 'RW', 'AptRank1']


    labels = []
    results = ddict(list)

    performaceMetricsOutput = ['AUC', 'pAUC', 'AUC_bootstrapMedian',
                               'pAUC_bootstrapMedian', 'AUC_PR', 'pAUC_PR',
                               'AUC_PR_bootstrapMedian',
                               'pAUC_PR_bootstrapMedian', 'F1', 'predictionsN',
                               'truePositiveN', 'truePositiveNtopPercent']

    # Trim iteratively the nodes which have degree = 1 in the last input network
    # (Assuming the last input network is the largest one)
    _net, removedNodes = generateDegree1net(conns[-1], mappingFile)

    if experiment.startswith('D') or experiment=='test':
        inputListFile = paths.PROJECT_DIR + '/data/prospective_input_list/top10DieasesFormatedmappedFiltered.txt'
    elif experiment.startswith('G'):
        inputListFile = paths.PROJECT_DIR + '/data/prospective_input_list/interestingGenes_ratioFormatedmappedFiltered.txt'
    inputList = pd.read_csv(inputListFile, sep="\t", header=None)[0].values
    inputListInitialNames = pd.read_csv(
        inputListFile, sep="\t", header=None)[4].values
    # inputIDtoInitial = {inputList[i]: inputListInitialNames[i] for i in range(len(inputList))}
    inputPrefix = inputListFile.split('/')[-1].split('.')[0]

    FDAdrugsFile = paths.PROJECT_DIR + '/data/other/full database_12132016_FDAdrugsFormatedmappedFiltered.txt'
    MeSH_name_file1 = paths.PROJECT_DIR + '/data/other/c2016.bin_dict'
    MeSH_name_file2 = paths.PROJECT_DIR + '/data/other/d2016.bin_dict'

    # inputList = inputList[0:2]
    # inputListInitialNames = inputListInitialNames[0:2]
    print inputList, inputListInitialNames
    for algorithm, algorithmName in zip(algorithms, algorithmNames):
        for i in range(len(networkNames)):
            currQueryMode = selectedQueryModes[i]
            currPredictionMode = selectedPredictionModes[i]
            print 'Running Prospective for {}'.format(experiment)
            currNet, _removedNodes = generateDegree1net(
                conns[i], mappingFile, removedNodes)
            currName = networkNames[i]
            later = time()
            print('Graph generation took {} min'.format((later - previous) / 60))
            previous = time()
            ###################################################################
            print(currNet)
            print('Generating prospective object for {}'.format(currName))
            validation = prospective(name=inputPrefix, algorithm=algorithm, network=currNet,
                                     selectedEntityList=inputList, selectedEntityInitName=inputListInitialNames,
                                     selectedPredictionMode=currPredictionMode,
                                     selectedQueryMode=currQueryMode,
                                     diff_type=diff_type,
                                     mappingFile=mappingFile, FDAdrugsFile=FDAdrugsFile,
                                     MeSH_name_file1=MeSH_name_file1, MeSH_name_file2=MeSH_name_file2,
                                     splitPerformanceByEntity=True)
            validation.perform()
            currentResult = validation.getResults()
            # labels.append("%s_%s" % (algorithmName, networkName))
            # labels.append('{}'.format(currName))
            labels.append("%s (%s" % (algorithmName, currName))
            for x in (validation.savedVars):
                results[x].append(currentResult[x])

    ###########################################################################
    later = time()
    print('Validation overall took {} min'.format((later - previous) / 60))
    previous = time()
    outputPrefix = '_'.join(
        algorithmNames) + '_Prospective_' + experiment + '_final_excludeKnown'
    # os.chdir('../resultValidation')
    # ###########################################################################
    printPerformance(results, performaceMetricsOutput, labels, outputPrefix)

    print('\"Networking\" is awesome!!!')
    print('Fusing network cures cancer!!!')
    end = time()
    print('This program ran {} min'.format((end - start) / 60))
